@@ -1,12 +1,5 @@
-use rechannel::{
-    disconnect_packet,
-    error::{DisconnectionReason, RechannelError},
-    server::RechannelServer,
-};
+use crate::{RenetConnectionConfig, NUM_DISCONNECT_PACKETS_TO_SEND};
 
-use renetcode::{NetcodeServer, PacketToSend, ServerResult, NETCODE_KEY_BYTES, NETCODE_USER_DATA_BYTES};
-
-use log::error;
 use std::{
     collections::VecDeque,
     io,
@@ -14,8 +7,12 @@ use std::{
     time::Duration,
 };
 
-use crate::{RenetConnectionConfig, NUM_DISCONNECT_PACKETS_TO_SEND};
+use log::error;
+use rechannel::{disconnect_packet, error::DisconnectionReason, remote_connection::NetworkInfo, server::RechannelServer};
+use renetcode::{NetcodeServer, PacketToSend, ServerResult, NETCODE_KEY_BYTES, NETCODE_USER_DATA_BYTES};
 
+/// A server that can establish authenticated connections with multiple clients.
+/// Can send/receive encrypted messages from/to them.
 #[derive(Debug)]
 pub struct RenetServer {
     socket: UdpSocket,
@@ -25,26 +22,34 @@ pub struct RenetServer {
     events: VecDeque<ServerEvent>,
 }
 
+/// Events that can occur in the server.
 #[derive(Debug, Clone)]
-#[allow(clippy::large_enum_variant)] // TODO: Consider boxing types
 pub enum ServerEvent {
-    ClientConnected(u64, [u8; NETCODE_USER_DATA_BYTES]),
+    ClientConnected(u64, Box<[u8; NETCODE_USER_DATA_BYTES]>),
     ClientDisconnected(u64),
 }
 
+/// Configuration options for the renet server.
 pub struct ServerConfig {
+    /// Maximum numbers of clients that can be connected at a time
     pub max_clients: usize,
+    /// Unique identifier to this game/application
+    /// One could use a hash function with the game current version to generate this value.
+    /// So old version would be unable to connect to newer versions.
     pub protocol_id: u64,
-    pub server_addr: SocketAddr,
+    /// Publicly available address that clients will try to connect to. This is
+    /// the address used to generate the ConnectToken.
+    pub public_addr: SocketAddr,
+    /// Private key used for encryption in the server
     pub private_key: [u8; NETCODE_KEY_BYTES],
 }
 
 impl ServerConfig {
-    pub fn new(max_clients: usize, protocol_id: u64, server_addr: SocketAddr, private_key: [u8; NETCODE_KEY_BYTES]) -> Self {
+    pub fn new(max_clients: usize, protocol_id: u64, public_addr: SocketAddr, private_key: [u8; NETCODE_KEY_BYTES]) -> Self {
         Self {
             max_clients,
             protocol_id,
-            server_addr,
+            public_addr,
             private_key,
         }
     }
@@ -63,7 +68,7 @@ impl RenetServer {
             current_time,
             server_config.max_clients,
             server_config.protocol_id,
-            server_config.server_addr,
+            server_config.public_addr,
             server_config.private_key,
         );
 
@@ -86,6 +91,7 @@ impl RenetServer {
         self.events.pop_front()
     }
 
+    /// Disconnects a client.
     pub fn disconnect(&mut self, client_id: u64) {
         let server_result = self.netcode_server.disconnect(client_id);
         if let Err(e) = handle_server_result(server_result, &self.socket, &mut self.reliable_server, &mut self.events) {
@@ -93,12 +99,19 @@ impl RenetServer {
         }
     }
 
+    /// Disconnects all connected clients.
     pub fn disconnect_clients(&mut self) {
         for client_id in self.netcode_server.clients_id() {
             self.disconnect(client_id);
         }
     }
 
+    /// Returns the client's network info if the client exits.
+    pub fn network_info(&self, client_id: u64) -> Option<&NetworkInfo> {
+        self.reliable_server.network_info(client_id)
+    }
+
+    /// Advances the server by duration, and receive packets from the network.
     pub fn update(&mut self, duration: Duration) -> Result<(), io::Error> {
         loop {
             match self.socket.recv_from(&mut self.buffer) {
@@ -141,22 +154,32 @@ impl RenetServer {
         Ok(())
     }
 
+    /// Receive a message from a client over a channel.
     pub fn receive_message(&mut self, client_id: u64, channel_id: u8) -> Option<Vec<u8>> {
         self.reliable_server.receive_message(&client_id, channel_id)
     }
 
-    pub fn send_message(&mut self, client_id: u64, channel_id: u8, message: Vec<u8>) -> Result<(), RechannelError> {
-        self.reliable_server.send_message(&client_id, channel_id, message)
+    /// Verifies if a message can be sent to a client over a channel.
+    pub fn can_send_message(&self, client_id: u64, channel_id: u8) -> bool {
+        self.reliable_server.can_send_message(&client_id, channel_id)
     }
 
+    /// Send a message to a client over a channel.
+    pub fn send_message(&mut self, client_id: u64, channel_id: u8, message: Vec<u8>) {
+        self.reliable_server.send_message(&client_id, channel_id, message);
+    }
+
+    /// Send a message to all client, except the specified one, over a channel.
     pub fn broadcast_message_except(&mut self, client_id: u64, channel_id: u8, message: Vec<u8>) {
         self.reliable_server.broadcast_message_except(&client_id, channel_id, message)
     }
 
+    /// Send a message to all client over a channel.
     pub fn broadcast_message(&mut self, channel_id: u8, message: Vec<u8>) {
         self.reliable_server.broadcast_message(channel_id, message);
     }
 
+    /// Send packets to connected clients.
     pub fn send_packets(&mut self) -> Result<(), io::Error> {
         for client_id in self.reliable_server.connections_id().into_iter() {
             let packets = match self.reliable_server.get_packets_to_send(&client_id) {
@@ -180,6 +203,7 @@ impl RenetServer {
         Ok(())
     }
 
+    /// Returns all the connected clients id.
     pub fn clients_id(&self) -> Vec<u64> {
         self.netcode_server.clients_id()
     }
